import { INFTMarkerInfo, IImageObj, INFTMarker } from "./abstractions/CommonInterfaces";
import { AbstractARControllerNFT } from "./abstractions/AbstractARControllerNFT";
export declare class ARControllerNFT implements AbstractARControllerNFT {
    private id;
    private _width;
    private _height;
    private _cameraParam;
    private cameraId;
    private artoolkitNFT;
    private FS;
    private StringList;
    private listeners;
    private nftMarkers;
    private transform_mat;
    private transformGL_RH;
    private camera_mat;
    private videoWidth;
    private videoHeight;
    private videoSize;
    private framesize;
    private videoLuma;
    private grayscaleEnabled;
    private grayscaleSource;
    private nftMarkerFound;
    private nftMarkerFoundTime;
    private nftMarkerCount;
    private defaultMarkerWidth;
    private _bwpointer;
    constructor();
    constructor(width: number, height: number);
    constructor(width: number, height: number, cameraParam: string);
    static initWithDimensions(width: number, height: number, cameraParam: string): Promise<ARControllerNFT>;
    static initWithImage(image: IImageObj, cameraParam: string): Promise<ARControllerNFT>;
    static customInit(width: number, height: number, cameraParam: string, callback: () => void): Promise<ARControllerNFT>;
    set width(width: number);
    get width(): number;
    set height(height: number);
    get height(): number;
    set cameraParam(cameraParam: string);
    get cameraParam(): string;
    process(image: IImageObj): void;
    detectNFTMarker(): number;
    trackNFTMarkerId(id: number, markerWidth?: number): INFTMarker;
    detectMarker(image: IImageObj): number;
    getNFTMarker(markerIndex: number): INFTMarkerInfo;
    getNFTData(index: number): object;
    addEventListener(name: string, callback: object): void;
    removeEventListener(name: string, callback: object): void;
    dispatchEvent(event: {
        name: string;
        target: any;
        data?: object;
    }): void;
    debugSetup(): void;
    transMatToGLMat(transMat: Float64Array, glMat: Float64Array, scale?: number): Float64Array;
    arglCameraViewRHf(glMatrix: Float64Array, glRhMatrix?: Float64Array, scale?: number): Float64Array;
    getTransformationMatrix(): Float64Array;
    getCameraMatrix(): Float64Array;
    setDebugMode(mode: boolean): number;
    getDebugMode(): boolean;
    getProcessingImage(): number;
    setLogLevel(mode: boolean): number;
    getLogLevel(): number;
    setProjectionNearPlane(value: number): void;
    getProjectionNearPlane(): number;
    setProjectionFarPlane(value: number): void;
    getProjectionFarPlane(): number;
    setThresholdMode(mode: number): number;
    getThresholdMode(): number;
    setThreshold(threshold: number): number;
    getThreshold(): number;
    loadNFTMarker(urlOrData: string, onSuccess: (ids: number) => void, onError: (err: number) => void): Promise<number[]>;
    loadNFTMarkers(urlOrData: Array<string>, onSuccess: (ids: number[]) => void, onError: (err: number) => void): Promise<number[]>;
    setImageProcMode(mode: number): number;
    getImageProcMode(): number;
    setGrayData(data: Uint8Array): void;
    private converter;
    private _initialize;
    private _initNFT;
    private _copyImageToHeap;
}
